// $Id$
//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2002-2003 Andreas Huber Doenni, Switzerland
// Permission to copy, use, modify, sell and distribute this software
// is granted provided this copyright notice appears in all copies.
// This software is provided "as is" without express or implied
// warranty, and with no claim as to its suitability for any purpose.
//////////////////////////////////////////////////////////////////////////////

//=============================================================================
/**
 *  @file  state_machine_task.h
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_STATE_MACHINE_TASK_H
#define JYUGEM_BEE_STATE_MACHINE_TASK_H

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#endif

#include <boost/config.hpp>  // BOOST_STATIC_CONSTANT
#include <boost/fsm/event.hpp>
#include <boost/fsm/detail/rtti_policy.hpp>
#include <boost/fsm/exception_translator.hpp>
#include <boost/fsm/result.hpp>
#include <boost/fsm/state_machine.hpp>
#include <boost/fsm/state.hpp>
#include <boost/fsm/simple_state.hpp>
#include <boost/fsm/transition.hpp>
#include <boost/fsm/custom_reaction.hpp>
#include <boost/fsm/deferral.hpp>
#include <boost/fsm/shallow_history.hpp>
#include <boost/fsm/deep_history.hpp>

#include <boost/fsm/detail/state_base.hpp>
#include <boost/fsm/detail/constructor.hpp>

#include <boost/mpl/list.hpp>
#include <boost/mpl/clear.hpp>

#include <boost/intrusive_ptr.hpp>
#include <boost/type_traits/is_pointer.hpp>
#include <boost/noncopyable.hpp>
#include <boost/assert.hpp>
#include <boost/cast.hpp>  // boost::polymorphic_downcast

#include <boost/mpl/list.hpp>
#include <boost/format.hpp>

namespace fsm = boost::fsm;
namespace mpl = boost::mpl;

#include "jyugem/bee/task.h"
using namespace jyugem::bee;

//
// Common Event
//
struct EvEvalCondition : fsm::event< EvEvalCondition > {};

namespace jyugem { namespace bee
{

class task;
class trigger;

class BEE_Export state_machine_task_base : nameable
{
public:
  state_machine_task_base(const std::string& name, bee::task * tsk)
    : nameable(name), m_task(tsk) {}
  virtual ~state_machine_task_base() {}

  virtual int send(trigger * trig);  // request from other task
  int changed(trigger * trig);   // request to other tasks

  template<class T>
  T * cast_task()
  {
    T * result = null;
    try
    {
      result = dynamic_cast<T *>(m_task);
    }
    catch (const std::bad_cast& ex)
    {
      TRACE_FAIL((_TX("Failure cast: %s\n"), ex.what()));
    }
    return result;
  }

protected:
  bee::task * m_task;

  ACE_Thread_Mutex m_stlock;    // mutex for state_machine thread
};


//////////////////////////////////////////////////////////////////////////////
template< class MostDerived, class InitialState >
class state_machine_task
  : public fsm::state_machine< MostDerived, InitialState,
    std::allocator< void >, fsm::exception_translator<> >,
    public state_machine_task_base
{
  typedef fsm::state_machine< MostDerived, InitialState, std::allocator< void >,
    fsm::exception_translator<> > machine_base;
public:

  typedef fsm::detail::rtti_policy rtti_policy_type;
  typedef fsm::event_base event_base_type;
  typedef boost::intrusive_ptr< const event_base_type > event_base_ptr_type;

  //////////////////////////////////////////////////////////////////////////
  state_machine_task(const std::string& name, bee::task * task)
    : state_machine_task_base(name, task)
  {
    MostDerived::InitEventDictionary();
  }

  typedef fsm::detail::state_base< std::allocator< void >, rtti_policy_type >
    state_base_type;
  typedef mpl::clear< mpl::list<> >::type context_type_list;
  typedef std::map<std::string, event_base_type * , std::less<std::string> >
    event_dictionary;

  event_base_type * get_event(const std::string& eid)
  {
    event_dictionary::iterator iter = MostDerived::_event_dictionary.find(eid);
    if (iter == MostDerived::_event_dictionary.end())
    {
      throw failure(FORMATER1("Event not found(%1%)", eid));
    }
    return iter->second;
  }

  // Transition by trigger
  template < class LeafState >
  int send_trigger(const std::string& trigger)
  {
    try
    {
      event_base_type * eid = get_event(trigger);
      m_LastEvent = trigger;
      return this->send_event( *eid );
    }
    catch (const failure& ex)
    {
      TRACE_FAIL((_TX("%s\n"), ex.what()));
    }
    return -1;
  }

  int send_event(const event_base_type & evt)
  {
    const BCHAR * current = this->current_state();
    if (current == null)
    {
      //return -1;
    }
    m_PreviousState = current;
    this->process_event(evt);
    return 0;
  }

  // Get current state name
  template < class LeafState >
  const cstr_id * current_state_name()
  {
    BEE_GUARD_LOOP(ACE_Thread_Mutex, ace_mon, m_stlock);

    const cstr_id * st = null;
    try
    {
      const LeafState& leaf = this->state_cast< const LeafState& >();
      st = const_cast< LeafState& >(leaf).m_nameptr;
      return st;
    }
    catch ( const std::bad_cast & )
    {
      std::cout << "current state is <null>\n";
    }
    return st;
  }

  const cstr_id * get_current_state( int regionNum = 0 )
  {
    BEE_GUARD_LOOP(ACE_Thread_Mutex, ace_mon, m_stlock);

    const cstr_id * st = null;
    int orthogonalRegion = 0;
    for (state_iterator pLeafState = state_begin();
         pLeafState != state_end(); ++pLeafState)
    {
      const state_base_type * pState = &*pLeafState;
      while ( pState != 0 )
      {
        if ( pState == &*pLeafState )
        { // leaf state node
          st = pState->custom_dynamic_type_ptr< cstr_id >();
        }
        pState = pState->outer_state_ptr();
      }
      if (orthogonalRegion == regionNum)
      {
        // orthogonalRegion is the most new region that updated.
        break;
      }
      ++orthogonalRegion;
    }
    return st;
  }

  const BCHAR * current_state(int regionNum = 0)
  {
    const cstr_id * stp = get_current_state(regionNum);
    if (stp == null)
    {
      TRACE_FAIL((_TX("current state is <null>\n")));
      return null;
    }
    return stp->name();
  }

  bool prevState(const std::string& staname)
  {
    return (m_PreviousState == staname);
  }

  bool lastEvent(const std::string& evtname)
  {
    return (m_LastEvent == evtname);
  }

//------------------------------------------------------------------------
public:
  //////////////////////////////////////////////////////////////////////////
  // The following declarations should be private.
  // They are only public because many compilers lack template friends.
  //////////////////////////////////////////////////////////////////////////
  void terminate()
  {
    machine_base::terminate();
  }

  void terminate( state_machine_task & machine )
  {
    machine_base::terminate( machine );
  }

  void terminate( machine_base::state_base_type & theState )
  {
    machine_base::terminate( theState );
  }

protected:
  std::string m_LastEvent;
  std::string m_PreviousState;

};


} } // *namespace* jyugem::bee

#endif  // JYUGEM_BEE_STATE_MACHINE_TASK_H
