// $Id$

//=============================================================================
/**
 *  @file  variable_map.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================


#ifndef JYUGEM_BEE_VARIABLE_MAP_H
#define JYUGEM_BEE_VARIABLE_MAP_H

#include "nameable.h"

namespace jyugem { bee
{
//
// vector and map table
//
class variable_map
{
public:
  typedef map<std::string, variable_def *, less<std::string> > defines;
  typedef map<std::string, variable *, less<std::string> > variables;

  variable_map() {}
  variable_map(const variable_map& rhs)
    : m_defmap(rhs.m_defmap), m_defmap(rhs.m_defmap) {}
  ~variable_map() {}

  variable_map& operator=(const variable_map& rhs)
  {
    if (this == &rhs)
      return *this;
    m_defmap = rhs.m_defmap;
    m_varmap = rhs.m_varmap;
    return *this;
  }

  variable * find(const std::string& key)
  {
    variables::iterator iter = m_varmap.find(key);
    if (iter == m_varmap.end())
      return null;
    return (*iter).second;
  }

  int add(variable * var)
  {
    m_varmap.insert(std::make_pair(var->uniqueName(), var));
    return 0;
  }

  variable * remove(const std::string& key)
  {
    variables::iterator mapiter = m_varmap.find(key);
    if (mapiter == m_varmap.end())
      return null;

    variable * var = (*mapiter).second;
    m_varmap.erase(mapiter);
    return var;
  }

  // operation of variable_def
  variable * create(const std::string& key)
  {
    defines::iterator iter = m_defmap.find(key);
    if (iter == m_defmap.end())
      return null;
    variable_def * factory = (*iter).second;
    variable * var = factory->create();
    this->add(var);
    return var;
  }
  variable * create(const std::string& key, const std::string& base)
  {
    defines::iterator iter = m_defmap.find(key);
    if (iter == m_defmap.end())
      return null;
    variable_def * factory = (*iter).second;
    variable * var = factory->create(base);
    this->add(var);
    return var;
  }

  variable_def * find_def(const std::string& key)
  {
    defines::iterator iter = m_defmap.find(key);
    if (iter == m_defmap.end())
      return null;
    return (*iter).second;
  }

  int add_def(variable_def * def)
  {
    m_defmap.insert(std::make_pair(def->uniqueName(), def));
    return 0;
  }

  variable_def * remove_def(const std::string& key)
  {
    defines::iterator mapiter = m_defmap.find(key);
    if (mapiter == m_defmap.end())
      return null;

    variable_def * def = (*mapiter).second;
    m_defmap.erase(mapiter);
    return def;
  }


protected:
  defines   m_defmap;
  variables m_varmap;
};

} } // *namespace* - jyugem::bee

#endif  /* JYUGEM_BEE_VARIABLE_MAP_H */
