// $Id$

//=============================================================================
/**
 *  @file    cidrw.h
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================
#ifndef JYUGEM_CIDRW_H
#define JYUGEM_CIDRW_H

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#endif


#include "jyugem/bee/state_task.h"
#include <ctime>
#include <iostream>
#include <string>

namespace bee = jyugem::bee;


//
namespace jyugem { namespace cidrw
{


//
// E99 : CARRIER ID READER/WRITER FUNCTIONAL STANDARD
//

class CIDRWHead : public bee::state_task
{
  friend class CIDRW;
  friend class CIDRWHeadState;

public:
  // Condition
  enum
  {
    NO_ALARMS, NEEDS_MAINTENANCE, RW_FAULT, RW_RATE_FAULT, NO_POWER
  };
  enum { IDLE, BUSY, NO_OPERATING };  // Status


  CIDRWHead(const BCHAR * idp);
  // CIDRWHead(const CIDRWHead& rhs) { this->copy(); }
  ~CIDRWHead();

  //CIDRWHead& operator=(const CIDRWHead& rhs)
  //{
  //  if (this == &rhs)
  //     return *this;
  //  this->copy();
  //  return *this;
  //}

  int open0();
  
  const BCHAR * current_state();

protected:
  //void copy(const CIDRWHead& rhs);

protected:
  int           m_headStatus;
  std::string   m_id;
  uint          m_cycles;
  int           m_condition;
  bee::datetime m_installed;
  std::string   m_maintenanceData;

};


//
//
//
class CIDRW : public bee::state_task
{
  friend class CIDRWState;

public:
  enum { NO_ALRAMS, ALARMS };  // Alarm Status
  enum { IDLE, BUSY, MAINTENANCE };  // Operational Status


  CIDRW(int hc = 2);
  // CIDRW(const CIDRW& rhs) : bee::state_task(rhs.name()) { this->copy(rhs); }
  virtual ~CIDRW();

  // CIDRW& operator=(const CIDRW& rhs)
  // {
  //   if ( this== &rhs)
  //     return *this;
  //   this->copy(rhs);
  //   return *this;
  // }

  int open0();

  const BCHAR * current_state();

  const BCHAR * head_state(int hid);
  const int     update_head(int hid, const std::string& evtname);

protected:
  // void copy(const CIDRW& rhs);

public:
  int m_alarmStatus;
  int m_operationalStatus;
  bee::datetime m_installed;

  std::string m_Configuration;      // Number of heads
  std::string m_SoftwareRevisionLevel;
  std::string m_DeviceType;         // CarrierID Reader or CarrierID Reader/Writer
  std::string m_HardwareRevisionLevel;
  std::string m_MaintenanceData;
  std::string m_Manufacturer;
  std::string m_ModelNumber;
  std::string m_SerialNumber;
  
  std::vector<CIDRWHead *> m_heads;
};


} }  // *namespace* jyugem::cidrw

#endif  /* JYUGEM_CIDRW_H */

