// $Id$

//=============================================================================
/**
 *  @file    Carrier.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_CMS_CARRIER_H
#define JYUGEM_CMS_CARRIER_H

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#endif


#include "jyugem/bee/state_task.h"
#include <boost/mpl/list.hpp>
#include <boost/format.hpp>

#include "jyugem/bee/state_task.h"
namespace bee = jyugem::bee;


//
namespace jyugem { namespace cms
{

//
// E87 : SPECIFICATION FOR CARRIER MANAGEMENT
//

class Content
{
public:
  Content() {}
  Content(const std::string& lotid, const std::string& substid)
    : m_lotID(lotid), m_substID(substid) {}
  Content(const Content& rhs)
    : m_lotID(rhs.m_lotID), m_substID(rhs.m_substID) {}
  ~Content() {}

  Content& operator=(const Content& rhs)
  {
    if (this == &rhs)
      return *this;
    m_lotID = rhs.m_lotID;
    m_substID = rhs.m_substID;
    return *this;
  }

  std::string LotID()       { return m_lotID; }
  std::string SubstrateID() { return m_substID; }

  std::string m_lotID;
  std::string m_substID;
};


//------------------------------------------------------------------------------
// Carrier
//------------------------------------------------------------------------------
class Carrier : public bee::state_task
{
  friend class CarrierState;
  friend class CarrierClass;

public:
  Carrier();
  // Carrier(const Carrier& carrier);
  virtual ~Carrier();

  // Carrier& operator=(const Carrier& rhs)
  // {
  //   if ( this== &rhs)
  //     return *this;
  //   this->copy(rhs);
  //   return *this;
  // }

  // Outhgonal region of state
  enum Orthogonal { IDSTATUS = 0, SLOTMAP = 1, ACCESS = 2 };

  enum            // CarrierID Status
  {
    ID_NOT_READ,
    ID_WAITING_FOR_HOST,
    ID_VERIFICATION_OK,
    ID_VERIFICATION_FAILED
  };

  enum            // CarrierAccessing Status
  {
    NOT_ACCESSED,
    IN_ACCESS,
    CARRIER_COMPLETE,
    CARRIER_STOPPED
  };

  enum            // SlotMap Status
  {
    SLOTMAP_NOT_READ,
    SLOT_WAITING_FOR_HOST,
    SLOTMAP_VERIFICATION_OK,
    SLOTMAP_VERIFICATION_FAILED,
  };

  enum            // Slot Status
  {
    UNDEFINED,
    EMPTY,
    NOT_EMPTY,
    CORRECTLY_OCCUPIED,
    CROSS_SLOTTED,
    DOUBLE_SLOTTED
  };

  int open0();

  const BCHAR * current_state(Orthogonal region);

protected:
  // void copy(const Carrier& rhs);

public:
  bool m_BypassReadID;

  // Attribute data pointer in Object
  size_t m_Capacity;
  size_t m_SubstrateCount;
  int    m_CarrierIDStatus;
  int    m_CarrierAccessingStatus;
  int    m_SlotMapStatus;
  std::string m_LocationID;
  std::string m_Usage;
  std::vector<Content *> m_ContentMap;
  std::vector<int>       m_SlotMap;
  //
};

} }  // *namespace* jyugem::cms

#endif  // JYUGEM_CMS_CARRIER_H
