// $Id$

//=============================================================================
/**
 *  @file    bee_wrapper.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#include "jyugem/bee.h"

#include <boost/python/class.hpp>
#include <boost/python/module.hpp>
#include <boost/python/def.hpp>
#include <boost/python/suite/indexing/vector_indexing_suite.hpp>

using namespace jyugem::bee;
using namespace boost::python;

namespace jyugem { namespace bee
{

struct bee_wrapper
{
  static void wrap()
  {
    // Compile check only...
    class_<std::vector<std::string> >("StringVec")
        .def(vector_indexing_suite<std::vector<std::string> >())
    ;

    class_<std::vector<bee::byte> >("ByteVec")
        .def(vector_indexing_suite<std::vector<bee::byte> >())
    ;

    class_<std::vector<short> >("ShortVec")
        .def(vector_indexing_suite<std::vector<short> >())
    ;

    class_<std::vector<int> >("IntVec")
        .def(vector_indexing_suite<std::vector<int> >())
    ;

    class_<std::vector<double> >("RealVec")
        .def(vector_indexing_suite<std::vector<double> >())
    ;

    class_<std::vector<bool> >("BoolVec")
        .def(vector_indexing_suite<std::vector<bool> >())
    ;
  }
};

struct bee_value_wrapper
{
  // A function set a value object as an argument.
  int convert(value& v, const std::string& str)
  {
    return v.set(str);
  }

  static void wrap()
  {
    class_< value >("value" )
      .def("set", convert)
      .def("toString", &value::toString)
      ;

    class_< Values >("Values" )
      .def("get", &Values::get)
      .def("add", &Values::add)
      ;
  }
};


struct bee_id_wrapper
{
  int convert(id& v, const std::string& str)
  {
    return v.set(str);
  }
  static void wrap()
  {
    class_< id >("id" )
      .def("set", convert)
      .def("toString", &id::toString)
      ;

    class_< Ids >("Ids" )
      .def("get", &Ids::get)
      .def("add", &Ids::add)
      ;
  }
};

//
//
//
void init_module()
{
  bee_wrapper::wrap();
  bee_value_wrapper::wrap();
  bee_id_wrapper::wrap();
}

} } // *namespace* jyugem::bee

BOOST_PYTHON_MODULE(bee)
{

  jyugem::bee::init_module();

}