// $Id$

//////////////////////////////////////////////////////////////////////////////
// Copyright (c) Andreas Huber Doenni 2002-2004.
// Use, modification and distribution are subject to the Boost Software
// License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
//////////////////////////////////////////////////////////////////////////////

//=============================================================================
/**
 *  @file    CarrierState.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#pragma warning( disable: 4244 ) // '__w64 int'
#endif

#include "CarrierState.h"
#include "jyugem/cms/Carrier.h"

namespace bee = jyugem::bee;
namespace fsm = boost::fsm;
namespace mpl = boost::mpl;

//
namespace jyugem { namespace cms
{

//-----------------------------------------------------------------------------
// State Machine
//
CarrierState::CarrierState(Carrier * carrier)
    : task_base(carrier->uniqueName(), carrier),
      m_verification(STATUS_UNKNOWN), m_idread(STATUS_UNKNOWN),
      m_slotmap_verification(STATUS_UNKNOWN)
{
}

CarrierState::task_base::event_dictionary  CarrierState::_event_dictionary;

static cstr_id _initCarrierStatus_( "InitCarrierStatus", 1 );
static cstr_id _idStatus_       ( "IDStatus", 2 );
static cstr_id _slotMapStatus_  ( "SlotMapStatus", 3 );
static cstr_id _accessingStatus_( "AccessingStatus", 4 );
static cstr_id _initIDStatus_   ( "InitIDStatus", 5 );
static cstr_id _idNotRead_      ( "IDNotRead", 6 );
static cstr_id _waitingForHost_ ( "WaitingForHost", 7 );
static cstr_id _slotMapNotRead_ ( "SlotMapNotRead", 8 );
static cstr_id _notAccessed_    ( "NotAccessed", 9 );
static cstr_id _inAccess_       ( "InAccess", 10 );
static cstr_id _carrierComplete_( "CarrierComplete", 11 );
static cstr_id _carrierStopped_ ( "CarrierStopped", 12 );
static cstr_id _idVerificationOK_       ( "IDVerificationOK", 13 );
static cstr_id _idVerificationFailed_   ( "IDVerificationFailed", 14 );
static cstr_id _slotMapWaitingForHost_  ( "SlotMapWaitingForHost", 15 );
static cstr_id _slotMapVerificationOK_  ( "SlotMapVerificationOK", 16 );
static cstr_id _slotMapVerificationFail_( "SlotMapVerificationFail", 17 );


void CarrierState::InitEventDictionary()
{
  if (! _event_dictionary.empty())
    return ;

  _event_dictionary.insert(std::make_pair("access",   new EvAccess()));
  _event_dictionary.insert(std::make_pair("bind",     new EvBind()));
  _event_dictionary.insert(std::make_pair("notify",   new EvCarrierNotification()));
  _event_dictionary.insert(std::make_pair("recreate", new EvCarrierReCreate()));
  _event_dictionary.insert(std::make_pair("relese",   new EvCarrierRelease()));
  _event_dictionary.insert(std::make_pair("abnomal",  new EvAbnormal()));
  _event_dictionary.insert(std::make_pair("normal",   new EvNormal()));
  _event_dictionary.insert(std::make_pair("proceed",  new EvProceedWithCarrier()));
  _event_dictionary.insert(std::make_pair("verify-ok", new EvVerificationOK()));
  _event_dictionary.insert(std::make_pair("read-id",   new EvRead()));
  _event_dictionary.insert(std::make_pair("read-map",  new EvSlotMapRead()));
  _event_dictionary.insert(std::make_pair("receive",   new EvReceiveCarrier()));
  _event_dictionary.insert(std::make_pair("tag-write", new EvCarrierTagWriteData()));
  _event_dictionary.insert(std::make_pair("verify-host", new EvHostVerification()));
  _event_dictionary.insert(std::make_pair("verify-fail", new EvVerificationFail()));
  _event_dictionary.insert(std::make_pair("carrier-in",  new EvCarrierIn()));
  _event_dictionary.insert(std::make_pair("carrier-out", new EvCarrierOut()));
  _event_dictionary.insert(std::make_pair("cancel-bind", new EvCancelBind()));
  _event_dictionary.insert(std::make_pair("cancel-carrier",new EvCancelCarrier()));
  _event_dictionary.insert(std::make_pair("cancel-port",   new EvCancelCarrierAtPort()));
  _event_dictionary.insert(std::make_pair("cancel-notify", new EvCancelCarrierNotification()));
  _event_dictionary.insert(std::make_pair("cancel-out",    new EvCancelCarrierOut()));
  _event_dictionary.insert(std::make_pair("position-error",new EvAbnormalPosition()));

  // Initialize static name of states
  InitCarrierStatus::custom_static_type_ptr( &_initCarrierStatus_ );
  IDStatus::       custom_static_type_ptr( &_idStatus_ );
  SlotMapStatus::  custom_static_type_ptr( &_slotMapStatus_ );
  AccessingStatus::custom_static_type_ptr( &_accessingStatus_ );
  InitIDStatus::   custom_static_type_ptr( &_initIDStatus_ );
  IDNotRead::      custom_static_type_ptr( &_idNotRead_ );
  WaitingForHost:: custom_static_type_ptr( &_waitingForHost_ );
  SlotMapNotRead:: custom_static_type_ptr( &_slotMapNotRead_ );
  NotAccessed::    custom_static_type_ptr( &_notAccessed_ );
  InAccess::       custom_static_type_ptr( &_inAccess_ );
  CarrierComplete::custom_static_type_ptr( &_carrierComplete_ );
  CarrierStopped:: custom_static_type_ptr( &_carrierStopped_ );
  IDVerificationOK::custom_static_type_ptr( &_idVerificationOK_ );
  IDVerificationFailed::custom_static_type_ptr( &_idVerificationFailed_ );
  SlotMapWaitingForHost::custom_static_type_ptr( &_slotMapWaitingForHost_ );
  SlotMapVerificationOK::custom_static_type_ptr( &_slotMapVerificationOK_ );
  SlotMapVerificationFail::custom_static_type_ptr( &_slotMapVerificationFail_ );

}

//
const cstr_id * CarrierState::get_current_name(int orthogonal)
{
  const cstr_id * result = null;
  if (orthogonal > Carrier::Orthogonal::ACCESS)
  {
    return result;
  }
  switch (orthogonal)
  {
  case Carrier::Orthogonal::IDSTATUS :
    result = this->current_state_name<ViewedState>();
    break;
  case Carrier::Orthogonal::SLOTMAP :
    result = this->current_state_name<SlotViewed>();
    break;
  case Carrier::Orthogonal::ACCESS :
    result = this->current_state_name<AccessViewed>();
    break;
  }
  return result;
}

} }  // *namespace* jyugen::cms

